@@ -315,8 +315,7 @@ class CLI(object):
         posted, reply = self._create_stack(stack_description, args)
         if posted:
             try:
-                reply = json.loads(reply)
-                return reply['text']
+                return self._format_output(json.loads(reply))
             except ValueError as e:
                 logging.error(reply)
 
@@ -364,12 +363,14 @@ class CLI(object):
 
         # Each additional row should include the actual data.
         for uuid in json_data.keys():
-            for c in json_data[uuid]['connectors']:
-                connectors.append(c)
-
-            backends = json_data[uuid]['backends']
+            csstore = []
             bstore = []
             cstore = []
+
+            for c in json_data[uuid]['connectors']:
+                csstore.append(c)
+
+            backends = json_data[uuid]['backends']
             for b in backends:
                 if b['storage']:
                     bstore.append(b['storage'])
@@ -382,6 +383,7 @@ class CLI(object):
                     cstore.append(' ')
             storage.append(bstore)
             compute.append(cstore)
+            connectors.append(csstore)
 
             status.append(json_data[uuid]['status'])
             base.append(json_data[uuid]['base'])
@@ -509,8 +511,6 @@ class CLI(object):
             elif connector_id == cg['uniq']:
                 connector_ip = cg['entry']['ip']
                 break
-            else:
-                logging.warning("no match: %s %s" % (connector_id, cg['uniq']))
 
         # Now form the ssh command. This just executes in the same shell. 
         if connector_ip:
@@ -522,6 +522,8 @@ class CLI(object):
             cmd = "ssh %s %s %s %s" % (key_opt, host_opt, ident, dest)
             logging.warning(cmd)
             os.execv('/usr/bin/ssh', cmd.split())
+        else:
+            logging.warning("could not find connector %s" % connector_id)
 
     def _parse_deploy_arg(self, param, args, default):
         pattern = re.compile('--%s=(\w+)' % param)
@@ -592,15 +594,21 @@ class CLI(object):
 
         return json_text
 
-    def _read_key_dir(self, root=False):
+    def _read_key_dir(self, options=None, root=False, server=False):
         """
         Read the location of the directory containing the keys
         used to communicate with the containers. 
         """
         if root:
-            keydir = ferry.install.DEFAULT_ROOT_KEY
+            keydir = ferry.install._get_key_dir(root=True, server=server)
         else:
-            keydir = ferry.install.DEFAULT_DOCKER_KEY
+            keydir = ferry.install._get_key_dir(root=False, server=server)
+
+        # May need to generate the user key. 
+        if not os.path.isfile(keydir):
+            self.installer._process_ssh_key(options=options, root=False)
+
+        # Read the key directory
         with open(keydir, 'r') as f: 
             k = f.read().strip().split("://")
             return k[1], k[0]
@@ -609,55 +617,66 @@ class CLI(object):
         """
         Check if the key being used is a temporary key. 
         """
-        keydir, tmp = self._read_key_dir(root=True)
+        keydir, tmp = self._read_key_dir(root=True, server=True)
         return tmp == "tmp"
 
-    def _check_ssh_key(self, root=False):
-        """
-        Check if the user has supplied a custom key
-        directory. If not copies over a temporary key. 
-        """
-        keydir, _ = self._read_key_dir(root)
-        if keydir == ferry.install.DEFAULT_KEY_DIR:
-            if root:
-                keydir = os.environ['HOME'] + '/.ssh/tmp-root'
-                ferry.install.GLOBAL_ROOT_DIR = 'tmp://' + keydir
-            else:
-                keydir = os.environ['HOME'] + '/.ssh/tmp-ferry'
-                ferry.install.GLOBAL_KEY_DIR = 'tmp://' + keydir
-
-            try:
-                os.makedirs(keydir)
-            except OSError as e:
-                if e.errno != errno.EEXIST:
-                    logging.error("Could not create ssh directory %s" % keydir)
-                    exit(1)
-            try:
-                shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa', keydir + '/id_rsa')
-                shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa.pub', keydir + '/id_rsa.pub')
+    def _copy_ssh_key(self, keydir):
+        try:
+            os.makedirs(keydir)
+        except OSError as e:
+            if e.errno != errno.EEXIST:
+                logging.error("Could not create ssh directory %s" % keydir)
+                exit(1)
+        try:
+            shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa', keydir + '/id_rsa')
+            shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa.pub', keydir + '/id_rsa.pub')
                             
-                uid = pwd.getpwnam(os.environ['USER']).pw_uid
-                gid = grp.getgrnam(os.environ['USER']).gr_gid
-                os.chown(keydir + '/id_rsa', uid, gid)
-                os.chmod(keydir + '/id_rsa', 0400)
-                os.chown(keydir + '/id_rsa.pub', uid, gid)
-                os.chmod(keydir + '/id_rsa.pub', 0444)
-            except OSError as e:
-                logging.error("Could not copy ssh keys (%s)" % str(e))
-                exit(1)
-            except IOError as e:
-                if e.errno == errno.EACCES:
-                    logging.error("Could not override keys in %s, please delete those keys and try again." % keydir)
+            uid = pwd.getpwnam(os.environ['USER']).pw_uid
+            gid = grp.getgrnam(os.environ['USER']).gr_gid
+            os.chown(keydir + '/id_rsa', uid, gid)
+            os.chmod(keydir + '/id_rsa', 0600)
+            os.chown(keydir + '/id_rsa.pub', uid, gid)
+            os.chmod(keydir + '/id_rsa.pub', 0644)
+        except OSError as e:
+            logging.error("Could not copy ssh keys (%s)" % str(e))
+            exit(1)
+        except IOError as e:
+            if e.errno == errno.EACCES:
+                logging.error("Could not override keys in %s, please delete those keys and try again." % keydir)
                 exit(1)
 
+    def _check_ssh_key(self, options=None, root=False, server=False):
+        """
+        Make sure that we pick the right ssh key and copy it over
+        if necessary. 
+        """
+        
+        if options and '-k' in options:
+            # The user wants to use a custom key. Just need
+            # to record the location of that key. 
             if root:
-                ferry.install._touch_file(ferry.install.DEFAULT_ROOT_KEY, 
+                ferry.install.GLOBAL_ROOT_DIR = 'user://' + options['-k'][0]
+                ferry.install._touch_file(ferry.install._get_key_dir(root=True, server=server), 
                                           ferry.install.GLOBAL_ROOT_DIR)
             else:
-                ferry.install._touch_file(ferry.install.DEFAULT_DOCKER_KEY, 
+                ferry.install.GLOBAL_KEY_DIR = 'user://' + options['-k'][0]
+                ferry.install._touch_file(ferry.install._get_key_dir(root=False, server=server), 
                                           ferry.install.GLOBAL_KEY_DIR)
-            logging.warning("Copied ssh keys to " + ferry.install.GLOBAL_KEY_DIR)
-
+        else:
+            # The user wants to use the default key.  
+            keydir, proto = self._read_key_dir(options=options, root=root)
+            if proto == "user" or keydir == ferry.install.DEFAULT_KEY_DIR:
+                if root:
+                    keydir = os.environ['HOME'] + '/.ssh/tmp-root'
+                    ferry.install.GLOBAL_ROOT_DIR = 'tmp://' + keydir
+                    ferry.install._touch_file(ferry.install._get_key_dir(root=True, server=server), 
+                                              ferry.install.GLOBAL_ROOT_DIR)
+                else:
+                    keydir = os.environ['HOME'] + '/.ssh/tmp-ferry'
+                    ferry.install.GLOBAL_KEY_DIR = 'tmp://' + keydir
+                    ferry.install._touch_file(ferry.install._get_key_dir(root=False, server=server), 
+                                              ferry.install.GLOBAL_KEY_DIR)
+                self._copy_ssh_key(keydir)
 
     def _find_installed_app(self, app):
         """
@@ -676,12 +695,18 @@ class CLI(object):
                         if app == user + '/' + os.path.splitext(item)[0]:
                             return DEFAULT_FERRY_APPS + '/' + user + '/' + item
 
+    def _format_output(self, reply):
+        output = reply['text'] + "\n"
+        for c in reply['msgs'].keys():
+            output += "%s: %s\n" % (c, reply['msgs'][c])
+        return output
+        
     def dispatch_cmd(self, cmd, args, options):
         """
         This is the command dispatch table. 
         """
         if(cmd == 'start'):
-            self._check_ssh_key()
+            self._check_ssh_key(options=options)
 
             # Check if we need to build the image before running. 
             if '-b' in options:
@@ -713,7 +738,7 @@ class CLI(object):
                     elif reply['status'] == 'failed':
                         return 'could not create application'
                     else:
-                        return reply['text']
+                        return self._format_output(reply)
                 except ValueError as e:
                     logging.error(reply)
         elif(cmd == 'ps'):
@@ -729,12 +754,12 @@ class CLI(object):
             self.installer._stop_docker_daemon()
             return msg
         elif(cmd == 'clean'):
-            self._check_ssh_key()
-            self.installer.start_web()
+            self._check_ssh_key(options=options, server=True)
+            self.installer.start_web(clean=True)
             self.installer._clean_rules()
             self.installer.stop_web()
             self.installer._stop_docker_daemon(force=True)
-            self.installer._reset_ssh_key()
+            self.installer._reset_ssh_key(root=True)
             return 'cleaned ferry'
         elif(cmd == 'inspect'):
             return self._inspect_stack(args[0])
@@ -744,7 +769,7 @@ class CLI(object):
             self.installer.start_web(options)
             return 'started ferry'
         elif(cmd == 'ssh'):
-            self._check_ssh_key()
+            self._check_ssh_key(options=options)
             stack_id = args[0]
             connector_id = None
             if len(args) > 1:
@@ -752,7 +777,7 @@ class CLI(object):
 
             return self._connect_stack(stack_id, connector_id)
         elif(cmd == 'quit'):
-            self._check_ssh_key(root=True)
+            self._check_ssh_key(options=options, root=True, server=True)
             self._stop_all()
             self.installer.stop_web()
             self.installer._stop_docker_daemon()
@@ -761,7 +786,7 @@ class CLI(object):
                 self.installer._reset_ssh_key(root=True)
             return 'stopped ferry'
         elif(cmd == 'deploy'):
-            self._check_ssh_key()
+            self._check_ssh_key(options=options)
             stack_id = args.pop(0)
             return self._deploy_stack(stack_id, args)
         elif(cmd == 'ls'):
@@ -784,6 +809,9 @@ class CLI(object):
         elif(cmd == 'help'):
             return self._print_help()
         else:
+            # The user wants to perform some management function
+            # over the stack. 
+            self._check_ssh_key(options=options)
             stack_info = {'uuid' : args[0],
                           'action' : cmd}
             return self._manage_stacks(stack_info)
