@@ -36,14 +36,15 @@ class OpenMPIInitializer(object):
         """
         Start the service on the containers. 
         """
-        fabric.cmd(containers[:1], '/service/sbin/startnode %s %s %s' % (cmd, entry_point['mount'], 'master'))
-        fabric.cmd(containers[1:len(containers)], '/service/sbin/startnode %s %s %s' % (cmd, entry_point['mount'], 'slave'))
+        master_output = fabric.cmd(containers[:1], '/service/sbin/startnode %s %s %s' % (cmd, entry_point['mount'], 'glustermaster'))
+        slave_output = fabric.cmd(containers[1:len(containers)], '/service/sbin/startnode %s %s %s' % (cmd, entry_point['mount'], 'glusterslave'))
+        return dict(master_output.items() + slave_output.items())
     def start_service(self, containers, entry_point, fabric):
-        self._execute_service(containers, entry_point, fabric, "start")
+        return self._execute_service(containers, entry_point, fabric, "start")
     def restart_service(self, containers, entry_point, fabric):
-        self._execute_service(containers, entry_point, fabric, "restart")
+        return self._execute_service(containers, entry_point, fabric, "restart")
     def stop_service(self, containers, entry_point, fabric):        
-        self._execute_service(containers, entry_point, fabric, "stop")
+        return self._execute_service(containers, entry_point, fabric, "stop")
 
     def _generate_config_dir(self, uuid):
         """
@@ -155,15 +156,18 @@ class OpenMPIInitializer(object):
                     entry_point['instances'].append([server['data_ip'], server['host_name']])
                     entry_point['hosts'].append([server['data_ip'], server['host_name']])
             else:
-                # This is the MPI client. Create a "hosts" file that contains the
+                # This is the MPI client. First check if there are any compute
+                # nodes (the MPI client can be used with just a raw GlusterFS
+                # configuration). If it does have a compute, create a "hosts" file that contains the
                 # IP addresses of the compute nodes. 
                 entry_point['ip'] = containers[0]['data_ip']
                 compute = self._find_mpi_compute(containers)
-                with open(new_config_dir + '/hosts', 'w+') as hosts_file:
-                    for c in compute['hosts']:
-                        hosts_file.write(c[0] + "\n")
+                if compute and 'hosts' in compute:
+                    with open(new_config_dir + '/hosts', 'w+') as hosts_file:
+                        for c in compute['hosts']:
+                            hosts_file.write(c[0] + "\n")
+                    self._generate_mca_params(config, new_config_dir)
 
-            self._generate_mca_params(config, new_config_dir)
             for c in containers:
                 config_files = new_config_dir + '/*'
                 config_dirs.append([c['container'],
