@@ -13,20 +13,20 @@
 # limitations under the License.
 #
 
+from ferry.docker.docker import DockerCLI
+from ferry.ip.client import DHCPClient
 import ferry.install
 import json
 import logging
+from subprocess import Popen, PIPE
 import time
 import yaml
-from subprocess import Popen, PIPE
-from ferry.docker.docker import DockerCLI
-from ferry.ip.client import DHCPClient
 
 class DockerFabric(object):
     def __init__(self, bootstrap=False):
         self.repo = 'public'
         self.docker_user = 'root'
-        self.cli = DockerCLI()
+        self.cli = DockerCLI(ferry.install.DOCKER_REGISTRY)
         self.bootstrap = bootstrap
 
         # Bootstrap mode means that the DHCP network
@@ -62,10 +62,7 @@ class DockerFabric(object):
         Read the location of the directory containing the keys
         used to communicate with the containers. 
         """
-        if self.bootstrap:
-            keydir = ferry.install.DEFAULT_ROOT_KEY
-        else:
-            keydir = ferry.install.DEFAULT_DOCKER_KEY
+        keydir = ferry.install._get_key_dir(root=self.bootstrap, server=True)
         with open(keydir, 'r') as f: 
             k = f.read().strip().split("://")
             return k[1], k[0]
@@ -280,7 +277,7 @@ class DockerFabric(object):
         all_output = {}
         for c in containers:
             output = self.cmd_raw(c.internal_ip, cmd)
-            all_output[c] = output.strip()
+            all_output[c.host_name] = output.strip()
         return all_output
 
     def cmd_raw(self, ip, cmd):
