@@ -24,8 +24,8 @@ from ferry.docker.docker import DockerInstance
 app = Flask(__name__)
 
 # Initialize the storage driver
-docker = DockerManager()
 installer = Installer()
+docker = DockerManager()
 
 """
 Fetch the current information for a particular filesystem. 
@@ -296,39 +296,54 @@ def _allocate_connectors(payload, backend_info):
     if 'connectors' in payload:
         connectors = payload['connectors']
         for c in connectors:
-            connector_type = c['personality']
-            
+            # Check number of instances.
+            num_instances = 1
+            if 'instances' in c:
+                num_instances = int(c['instances'])
+
             # Check if this connector type has already been pulled
             # into the local index. If not, manually pull it. 
+            connector_type = c['personality']
+            logging.warning("allocating %d instances of %s" % (num_instances, connector_type))
             if not installer._check_and_pull_image(connector_type):
                 # We could not fetch this connetor. Instead of 
                 # finishing, just return an error.
                 return False, connector_info, None
 
-            # Connector names are created by the user 
-            # to help identify particular instances. 
-            connector_name = None
-            if 'name' in c:
-                connector_name = c['name']
+            for i in range(num_instances):
+                # Connector names are created by the user 
+                # to help identify particular instances. 
+                if 'name' in c:
+                    connector_name = c['name']
+                    if num_instances > 1:
+                        connector_name = connector_name + "-" + str(i)
+                else:
+                    connector_name = None
 
-            args = {}
-            if 'args' in c:
-                args = c['args']
+                # Arguments are optional parameters defined by
+                # the user and passed to the connectors.
+                if 'args' in c:
+                    args = c['args']
+                else:
+                    args = {}
 
-            ports = []
-            if 'ports' in c:
-                ports = c['ports']
+                # The user can choose to expose ports on the connectors.                
+                if 'ports' in c:
+                    ports = c['ports']
+                else:
+                    ports = []
 
-            uuid, containers = docker.allocate_connector(connector_type = connector_type,
-                                                         backend = backend_info, 
-                                                         name = connector_name, 
-                                                         args = args,
-                                                         ports = ports)
-            connector_plan.append( { 'uuid' : uuid,
-                                     'containers' : containers,
-                                     'type' : connector_type, 
-                                     'start' : 'start' } )
-            connector_info.append(uuid)
+                # Now allocate the connector. 
+                uuid, containers = docker.allocate_connector(connector_type = connector_type,
+                                                             backend = backend_info, 
+                                                             name = connector_name, 
+                                                             args = args,
+                                                             ports = ports)
+                connector_plan.append( { 'uuid' : uuid,
+                                         'containers' : containers,
+                                         'type' : connector_type, 
+                                         'start' : 'start' } )
+                connector_info.append(uuid)
     return True, connector_info, connector_plan
 
 """
@@ -412,11 +427,17 @@ def _start_all_services(backend_plan, connector_plan):
         else:
             docker._restart_service(c['uuid'], c['containers'], c['type'])
 
+    # The connectors can optionally output msgs for the user.
+    # Collect them so that we can display them later. 
+    all_output = {}
     for c in connector_plan:
         if c['start'] == 'start':
-            docker.start_service(c['uuid'], c['containers'])
+            output = docker.start_service(c['uuid'], c['containers'])
+            all_output = dict(all_output.items() + output.items())
         else:
-            docker._restart_connectors(c['uuid'], c['containers'], c['backend'])
+            output = docker._restart_connectors(c['uuid'], c['containers'], c['backend'])
+            all_output = dict(all_output.items() + output.items())
+    return all_output
 
 def _allocate_new(payload):
     """
@@ -429,15 +450,17 @@ def _allocate_new(payload):
         success, connector_info, connector_plan = _allocate_connectors(payload, backend_info['uuids'])
 
         if success:
-            _start_all_services(backend_plan, connector_plan)
+            output = _start_all_services(backend_plan, connector_plan)
             uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
             reply['text'] = str(uuid)
+            reply['msgs'] = output
         else:
             # One or more connectors was not instantiated properly. 
             docker.cancel_stack(backend_info, connector_info)
             reply['status'] = 'failed'
     else:
         reply['questions'] = backend_info['query']
+
     return json.dumps(reply)
 
 def _allocate_stopped(payload):
@@ -448,13 +471,14 @@ def _allocate_stopped(payload):
     backend_info, backend_plan = _allocate_backend_from_stopped(payload)
     if backend_info['status'] == 'ok':
         connector_info, connector_plan = _allocate_connectors_from_stopped(payload, backend_info['uuids'])
-        _start_all_services(backend_plan, connector_plan)        
+        output = _start_all_services(backend_plan, connector_plan)        
         uuid = docker.register_stack(backends = backend_info,
                                      connectors = connector_info,
                                      base = base,
                                      uuid = payload['_file'])
         return json.dumps({'status' : 'ok',
-                           'text' : str(uuid)})
+                           'text' : str(uuid),
+                           'msgs' : output})
     else:
         return json.dumps({'status' : 'failed'})
 
@@ -466,10 +490,11 @@ def _allocate_snapshot(payload):
     if backend_info['status'] == 'ok':
         connector_info, connector_plan = _allocate_connectors_from_snapshot(payload, 
                                                                             backend_info['uuids'])
-        _start_all_services(backend_plan, connector_plan)
+        output = _start_all_services(backend_plan, connector_plan)
         uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
         return json.dumps({'status' : 'ok',
-                           'text' : str(uuid)})
+                           'text' : str(uuid),
+                           'msgs' : output })
     else:
         return json.dumps({'status' : 'failed'})
 
@@ -482,10 +507,11 @@ def _allocate_deployed(payload, params=None):
         connector_info, connector_plan = _allocate_connectors_from_deploy(payload, 
                                                                           backend_info['uuids'],
                                                                           params)
-        _start_all_services(backend_plan, connector_plan)
+        output = _start_all_services(backend_plan, connector_plan)
         uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
         return json.dumps({'status' : 'ok',
-                           'text' : str(uuid)})
+                           'text' : str(uuid),
+                           'msgs' : output })
 
 
 @app.route('/login', methods=['POST'])
