@@ -39,12 +39,58 @@ from ferry.docker.fabric import DockerFabric
 from string import Template
 from subprocess import Popen, PIPE
 
+def _get_download_url():
+    if 'DOWNLOAD_URL' in os.environ:
+        return os.environ['DOWNLOAD_URL']
+    else:
+        return "https://s3.amazonaws.com/opencore"
+    
 def _get_ferry_home():
     if 'FERRY_HOME' in os.environ:
         return os.environ['FERRY_HOME']
     else:
         return os.path.dirname(__file__)
 
+def _get_docker_registry():
+    if 'DOCKER_REGISTRY' in os.environ:
+        return os.environ['DOCKER_REGISTRY']
+    else:
+        return None
+
+def _get_ferry_dir(server):
+    """
+    Get the Ferry data directory. 
+    For now we need to keep the client/server synched,
+    (this is something we need to fix later). 
+    """
+    if 'FERRY_DIR' in os.environ:
+        return os.environ['FERRY_DIR']
+    else:
+        # We need to figure out if this is being called
+        # from the server or client. 
+        if server:
+            return '/var/lib/ferry'
+        else:
+            return os.environ['HOME'] + '/.ferry'
+
+def _get_ferry_scratch():
+    if 'FERRY_SCRATCH' in os.environ:
+        scratch_dir = os.environ['FERRY_SCRATCH']
+    else:
+        scratch_dir = os.path.join(_get_ferry_dir(server=True), 'scratch')
+
+    if not os.path.isdir(scratch_dir):
+        logging.warning("CREATING SCRATCH DIR: " + scratch_dir)
+        os.makedirs(scratch_dir)
+
+    return scratch_dir
+
+def _get_key_dir(root, server):
+    if root:
+        return _get_ferry_dir(server=server) + '/rootdir'
+    else:
+        return _get_ferry_dir(server=server) + '/keydir'
+
 def _get_ferry_user():
     uid = pwd.getpwnam("root").pw_uid
     gid = grp.getgrnam("docker").gr_gid
@@ -90,15 +136,21 @@ def _supported_python():
     return sys.version_info[0] == 2
 
 def _touch_file(file_name, content, root=False):
+    # Check if we need to create the parent dir.
+    # first. If the parent dir. doesn't exist, then
+    # "open" will throw an exception. 
+    if not os.path.isdir(os.path.dirname(file_name)):
+        os.makedirs(os.path.dirname(file_name))
+
     try:
         f = open(file_name, 'w+')
         f.write(content)
         f.close()
 
+        os.chmod(file_name, 0664)
         if root:
             uid, gid = _get_ferry_user()
             os.chown(file_name, uid, gid)
-            os.chmod(file_name, 0664)
     except IOError as e:
         logging.error("Could not create %s.\n" % file_name)
 
@@ -111,19 +163,18 @@ DEFAULT_DOCKER_LOGIN=os.environ['HOME'] + '/.ferry.docker'
 DEFAULT_DOCKER_REPO='ferry'
 GUEST_DOCKER_REPO='ferry-user'
 DEFAULT_FERRY_OWNER='ferry:docker'
+DOCKER_REGISTRY=_get_docker_registry()
 DOCKER_CMD='docker-ferry'
 DOCKER_SOCK='unix:////var/run/ferry.sock'
-DOCKER_DIR='/var/lib/ferry'
 DOCKER_PID='/var/run/ferry.pid'
+DOCKER_DIR=_get_ferry_dir(server=True)
 DEFAULT_TEMPLATE_DIR=FERRY_HOME + '/data/templates'
 DEFAULT_BUILTIN_APPS=FERRY_HOME + '/data/plans'
-DEFAULT_FERRY_APPS='/var/lib/ferry/apps'
-DEFAULT_MONGO_DB='/var/lib/ferry/mongo'
-DEFAULT_MONGO_LOG='/var/lib/ferry/mongolog'
-DEFAULT_REGISTRY_DB='/var/lib/ferry/registry'
-DEFAULT_DOCKER_LOG='/var/lib/ferry/docker.log'
-DEFAULT_DOCKER_KEY='/var/lib/ferry/keydir'
-DEFAULT_ROOT_KEY='/var/lib/ferry/rootdir'
+DEFAULT_FERRY_APPS=DOCKER_DIR + '/apps'
+DEFAULT_MONGO_DB=DOCKER_DIR + '/mongo'
+DEFAULT_MONGO_LOG=DOCKER_DIR + '/mongolog'
+DEFAULT_REGISTRY_DB=DOCKER_DIR + '/registry'
+DEFAULT_DOCKER_LOG=DOCKER_DIR + '/docker.log'
 
 class Installer(object):
     def __init__(self, cli=None):
@@ -172,11 +223,12 @@ class Installer(object):
     def _clean_rules(self):
         self.network.clean_rules()
 
-    def _reset_ssh_key(self, root=False):
+    def _reset_ssh_key(self, root):
         """
-        Reset the temporary ssh key. 
+        Reset the temporary ssh key. This function should only be
+        called from the server. 
         """
-        keydir, tmp = self.cli._read_key_dir(root)
+        keydir, tmp = self.cli._read_key_dir(root=root)
 
         # Only reset temporary keys. User-defined key directories
         # shouldn't be touched. 
@@ -187,13 +239,17 @@ class Installer(object):
         if root:
             global GLOBAL_ROOT_DIR
             GLOBAL_ROOT_DIR = 'tmp://' + DEFAULT_KEY_DIR
-            _touch_file(DEFAULT_ROOT_KEY, GLOBAL_ROOT_DIR, root=True)
+            _touch_file(_get_key_dir(root=True, server=True), GLOBAL_ROOT_DIR, root=True)
         else:
             global GLOBAL_KEY_DIR
             GLOBAL_KEY_DIR = 'tmp://' + DEFAULT_KEY_DIR
-            _touch_file(DEFAULT_DOCKER_KEY, GLOBAL_KEY_DIR, root=True)
+            _touch_file(_get_key_dir(root=False, server=True), GLOBAL_KEY_DIR, root=True)
         
     def _process_ssh_key(self, options, root=False):
+        """
+        Initialize the ssh key location. This method is used
+        when starting the ferry server. 
+        """
         if root:
             global GLOBAL_ROOT_DIR
             if options and '-k' in options:
@@ -201,7 +257,7 @@ class Installer(object):
             else:
                 GLOBAL_ROOT_DIR = 'tmp://' + DEFAULT_KEY_DIR
                 logging.warning("using key directory " + GLOBAL_ROOT_DIR)
-                _touch_file(DEFAULT_ROOT_KEY, GLOBAL_ROOT_DIR, root=True)
+                _touch_file(_get_key_dir(root=True, server=True), GLOBAL_ROOT_DIR, root=True)
         else:
             global GLOBAL_KEY_DIR
             if options and '-k' in options:
@@ -209,7 +265,7 @@ class Installer(object):
             else:
                 GLOBAL_KEY_DIR = 'tmp://' + DEFAULT_KEY_DIR
                 logging.warning("using key directory " + GLOBAL_KEY_DIR)
-                _touch_file(DEFAULT_DOCKER_KEY, GLOBAL_KEY_DIR, root=True)
+                _touch_file(_get_key_dir(root=False, server=False), GLOBAL_KEY_DIR, root=False)
 
     def install(self, args, options):
         # Check if the host is actually 64-bit. If not raise a warning and quit.
@@ -280,7 +336,7 @@ class Installer(object):
     def _check_all_images(self):
         not_installed = []
         images = ['mongodb', 'ferry-base', 'hadoop-base', 'hadoop', 'hadoop-client',
-                  'hive-metastore', 'gluster', 'openmpi', 'cassandra', 'cassandra-client', 
+                  'hive-metastore', 'gluster', 'openmpi', 'openmpi-client', 'cassandra', 'cassandra-client', 
                   'titan', 'spark']
         for i in images:
             if not self._check_image_installed("%s/%s" % (DEFAULT_DOCKER_REPO, i)):
@@ -320,8 +376,8 @@ class Installer(object):
             sys.exit(1)
 
         # Check if the user wants to use a specific key directory. 
-        self._process_ssh_key(options, root=True)
-
+        self._process_ssh_key(options=options, root=True)
+                               
         # Check if the user-application directory exists.
         # If not, create it. 
         try:
@@ -357,7 +413,7 @@ class Installer(object):
         self._clean_web()
 
         # Copy over the ssh keys.
-        self.cli._check_ssh_key(root=True)
+        self.cli._check_ssh_key(root=True, server=True)
 
         # Start the Mongo server. Create a new configuration and
         # manually start the container. 
@@ -527,7 +583,7 @@ class Installer(object):
                         self._tag_images(image, repo, images)
 
             # After building everything, get rid of the temp dir.
-            shutil.rmtree("/tmp/dockerfiles")
+            # shutil.rmtree("/tmp/dockerfiles")
         else:
             logging.error("ferry daemon not started")
 
@@ -550,7 +606,7 @@ class Installer(object):
                     self._tag_images(image, repo, images)
 
             # After building everything, get rid of the temp dir.
-            shutil.rmtree("/tmp/dockerfiles")
+            # shutil.rmtree("/tmp/dockerfiles")
         else:
             logging.error("ferry daemon not started")
 
@@ -575,7 +631,9 @@ class Installer(object):
         out_file = "/tmp/dockerfiles/" + f + "/Dockerfile"
         out = open(out_file, "w+")
         uid, gid = _get_ferry_user()
+        download_url = _get_download_url()
         changes = { "USER" : repo,
+                    "DOWNLOAD_URL" : download_url,
                     "DOCKER" : gid }
         for line in open(image_dir + '/' + f + '/Dockerfile', "r"):
             s = Template(line).substitute(changes)
@@ -584,14 +642,14 @@ class Installer(object):
 
     def _build_image(self, image, f, repo, built_images, recurse=False, build=False):
         base = self._get_base(f)
-        if recurse and base != "base":
+        if recurse and base != "ubuntu:14.04":
             image_dir = os.path.dirname(os.path.dirname(f))
             dockerfile = image_dir + '/' + base + '/Dockerfile'
             self._build_image(base, dockerfile, repo, built_images, recurse, build)
 
         if not image in built_images:
-            if base == "base":
-                self._pull_image(base, tag='latest')
+            if base == "ubuntu:14.04":
+                self._pull_image(base)
 
             built_images[image] = True
             self._compile_image(image, repo, os.path.dirname(f), build)
@@ -686,12 +744,35 @@ class Installer(object):
         cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' | grep none | awk \'{print $1}\' | xargs ' + DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' rmi'
         Popen(cmd, stdout=PIPE, shell=True)
 
+    def _is_parent_dir(self, pdir, cdir):
+        pdirs = pdir.split("/")
+        cdirs = cdir.split("/")
+
+        # Parent directory can never be longer than
+        # the child directory. 
+        if len(pdirs) > len(cdirs):
+            return False
+            
+        for i in range(0, len(pdirs)):
+            # The parent directory shoudl always match
+            # the child directory. Ignore the start and end
+            # blank spaces caused by "split". 
+            if pdirs[i] != "" and pdirs[i] != cdirs[i]:
+                return False
+
+        return True
+
     def _is_running_btrfs(self):
         logging.warning("checking for btrfs")
-        cmd = 'cat /etc/mtab | grep %s | awk \'{print $3}\'' % DOCKER_DIR
+        cmd = 'cat /etc/mtab | grep btrfs | awk \'{print $2}\''
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
-        return output.strip() == "btrfs"
-
+        if output.strip() != "":
+            dirs = output.strip().split("\n")
+            for d in dirs:
+                if self._is_parent_dir(d, DOCKER_DIR):
+                    return True
+        return False
+        
     def _start_docker_daemon(self, options=None):
         # Check if the docker daemon is already running
         try:
